using System;
using System.Data;
using System.Configuration;
using Vevo.Shared.DataAccess;

namespace Vevo.DataAccessLib.Cart
{
    /// <summary>
    /// Summary description for OrderTrackingAccess
    /// </summary>
    public static class OrderTrackingAccess
    {
        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }


        public static string Create(
            string orderID, string senderName, string senderEmail, string message )
        {
            string trackingID = DataAccess.ExecuteScalar(
                "INSERT INTO OrderTracking (OrderID, SenderName, SenderEmail, Message) " +
                "VALUES (@OrderID, @SenderName, @SenderEmail, @Message); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( orderID ),
                DataAccess.CreateParameterString( senderName ),
                DataAccess.CreateParameterString( senderEmail ),
                DataAccess.CreateParameterString( message ) );
            return trackingID;
        }

        public static DataTable GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TrackingID, OrderID, CreatedDate, SenderName, SenderEmail, Message " +
                "FROM OrderTracking " +
                CreateSortByText( sortBy ) + "; " );
            DataColumn[] keys = new DataColumn[1] { table.Columns["TrackingID"] };
            table.PrimaryKey = keys;
            return table;
        }

        public static DataTable GetByOrderID( string orderID, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT TrackingID, OrderID, CreatedDate, SenderName, SenderEmail, Message " +
                "FROM OrderTracking " + 
                "WHERE OrderID = @OrderID " +
                CreateSortByText( sortBy ) + "; ", 
                DataAccess.CreateParameterString( orderID ) );
            DataColumn[] keys = new DataColumn[1] { table.Columns["TrackingID"] };
            table.PrimaryKey = keys;
            return table;
        }

    }

}
