using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Web;
using System.Web.Caching;

using Vevo.Shared.Utilities;
using Vevo.Domain;
using Vevo.Base.Domain;

namespace Vevo.Base.Data.Caching
{
    public class DataCache : IDataCache
    {
        #region Private
        private string KeyPrefix = "Vevo_";
        private string KeySeparator = "___";
        private string ParameterSeparator = "/";

        private string _repositoryName;

        private Cache Cache
        {
            get
            {
                return HttpRuntime.Cache;
            }
        }

        private string CreateMethodString( string signature, object[] parameters )
        {
            string result = signature;

            for (int i = 0; i < parameters.Length; i++)
            {
                if (parameters[i] is IEnumerable
                    && parameters[i].GetType() != typeof( string ))
                    //&& !ConvertUtilities.ToString( parameters[i] ).Contains( "/" ))
                {
                    foreach (object item in ((IEnumerable) parameters[i]))
                    {
                        result += ParameterSeparator + HttpUtility.UrlEncode( ConvertUtilities.ToString( item ) );
                    }
                }
                else
                    result += ParameterSeparator + HttpUtility.UrlEncode( ConvertUtilities.ToString( parameters[i] ) );
            }

            return result;
        }

        private string CreateKey( string signature, object[] parameters )
        {
            return KeyPrefix + _repositoryName + KeySeparator +
                CreateMethodString( signature, parameters );
        }
        #endregion


        #region Public Constructors
        public DataCache( string repositoryName )
        {
            _repositoryName = repositoryName;
        }
        #endregion


        public bool CacheEnabled
        {
            get { return true; }
        }


        #region Public Methods
        public void ClearRepositoryCache()
        {
            StringCollection removeList = new StringCollection();

            // Get the list of items to be removed
            IDictionaryEnumerator enumerator = Cache.GetEnumerator();
            while (enumerator.MoveNext())
            {
                if (enumerator.Key.ToString().StartsWith( KeyPrefix + _repositoryName + KeySeparator ))
                    removeList.Add( enumerator.Key.ToString() );
            }

            // Remove from cache
            foreach (string key in removeList)
            {
                Cache.Remove( key );
            }
        }

        public object GetValue( string signature, ICacheDependencyUtility cacheDependecy, params object[] parameters )
        {
            if (cacheDependecy.getHasChanged)
            {
                ClearRepositoryCache();
            }

            string key = CreateKey( signature, parameters );

            object value = Cache[key];
            if (value is ICloneable)
                return ((ICloneable) value).Clone();
            else
                return value;
        }

        public void SetValue( string signature, object value, params object[] parameters )
        {
            if (value != null)
            {
                string key = CreateKey( signature, parameters );
                Cache[key] = value;
            }
        }
        #endregion

    }
}
