using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Base.Data.Caching;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Domain;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ShippingOptionTypeRepository : IShippingOptionTypeRepository
    {
        #region Private

        private const string RepositoryName = "ShippingOptionTypeRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetIDByNameSignature = "GetIDByName";
        private const string TableName = "ShippingOptionType";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private ShippingOptionTypeRepositoryNoCache _repository = new ShippingOptionTypeRepositoryNoCache();

        #endregion

        #region Constructors

        public ShippingOptionTypeRepository()
        {
        }

        public ShippingOptionTypeRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public ShippingOptionType GetOne( string shippingOptionTypeID )
        {
            ShippingOptionType result = (ShippingOptionType) _dataCache.GetValue(
                GetOneSignature, _cacheDependency, shippingOptionTypeID );

            if (result == null)
            {
                result = _repository.GetOne( shippingOptionTypeID );
                _dataCache.SetValue( GetOneSignature, result, shippingOptionTypeID );
            }

            return result;
        }

        public string GetIDByName( string typeName )
        {
            string result = (string) _dataCache.GetValue( GetIDByNameSignature, _cacheDependency, typeName );

            if (result == null)
            {
                result = _repository.GetIDByName( typeName );
                _dataCache.SetValue( GetIDByNameSignature, result, typeName );
            }

            return result;
        }

        #endregion

    }
}
