using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ShippingWeightRateRepository : IShippingWeightRateRepository
    {
        #region Private

        private const string RepositoryName = "ShippingWeightRateRepository";

        private const string GetOneSignature = "GetOne";
        private const string SearchShippingWeightRateItemSignature = "SearchShippingWeightRateItem";
        private const string GetAllByShippingIDSignature = "GetAllByShippingID";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string TableName = "ShippingWeightRate";
        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private ShippingWeightRateRepositoryNoCache _repository = new ShippingWeightRateRepositoryNoCache();

        #endregion

        #region Constructors

        public ShippingWeightRateRepository()
        {
        }

        public ShippingWeightRateRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion


        #region Public Methods

        public ShippingWeightRate Save( ShippingWeightRate shippingWeightRate )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ShippingWeightRate newShippingWeightRate = _repository.Save( shippingWeightRate );
            _dataCache.ClearRepositoryCache();
            return newShippingWeightRate;
        }

        public ShippingWeightRate GetOne( string shippingWeightRateID )
        {
            ShippingWeightRate result = (ShippingWeightRate) _dataCache.GetValue( 
                GetOneSignature, 
                _cacheDependency, 
                shippingWeightRateID );

            if (result == null)
            {
                result = _repository.GetOne( shippingWeightRateID );
                _dataCache.SetValue( GetOneSignature, result, shippingWeightRateID );
            }

            return result;
        }

        public void Delete( string shippingWeightRateID )
        {
            _repository.Delete( shippingWeightRateID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByShippingID( string shippingID )
        {
            _repository.DeleteByShippingID( shippingID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ShippingWeightRate> GetAllByShippingID( string shippingID, string sortBy )
        {
            IList<ShippingWeightRate> list = (IList<ShippingWeightRate>) _dataCache.GetValue(
                   GetAllByShippingIDSignature,
                   _cacheDependency,
                   shippingID,
                   sortBy
                   );

            if (list == null)
            {
                list = _repository.GetAllByShippingID( shippingID, sortBy );
                _dataCache.SetValue(
                    GetAllByShippingIDSignature,
                    list,
                    shippingID,
                    sortBy
                    );
            }

            return list;
        }

        public IList<ShippingWeightRate> SearchShippingWeightRateItem(
            string shippingID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchShippingWeightRateItemSignature,
                _cacheDependency,
                shippingID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchShippingWeightRateItem(
                    shippingID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchShippingWeightRateItemSignature,
                    result,
                    shippingID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ShippingWeightRate>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        #endregion
    }
}
