using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using System.Web.UI;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ShippingZoneGroupRepository : IShippingZoneGroupRepository
    {
        #region Private

        private const string RepositoryName = "ShippingZoneGroupRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetShippingZoneItemByZoneGroupIDSignature = "GetShippingZoneItemByZoneGroupID";
        private const string GetOneShippingZoneItemSignature = "GetOneShippingZoneItem";
        private const string SearchShippingZoneItemSignature = "SearchShippingZoneItem";
        private const string GetShippingZoneItemTableSchemaSignature = "GetShippingZoneItemTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string SearchShippingZoneGroupSignature = "SearchShippingZoneGroup";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "ShippingZoneGroup", "ShippingZoneItem" } );

        private ShippingZoneGroupRepositoryNoCache _repository = new ShippingZoneGroupRepositoryNoCache();

        #endregion

        #region Constructors

        public ShippingZoneGroupRepository()
        {
        }

        public ShippingZoneGroupRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion


        #region IShippingZoneGroupRepository Members

        public IList<ShippingZoneItem> GetShippingZoneItemByZoneGroupID( string zoneGroupID )
        {
            IList<ShippingZoneItem> list =
                (IList<ShippingZoneItem>) _dataCache.GetValue(
                GetShippingZoneItemByZoneGroupIDSignature,
                _cacheDependency,
                zoneGroupID );

            if (list == null)
            {
                list = _repository.GetShippingZoneItemByZoneGroupID( zoneGroupID );
                _dataCache.SetValue( GetShippingZoneItemByZoneGroupIDSignature, list, zoneGroupID );
            }

            return list;
        }

        public IList<ShippingZoneItem> SearchShippingZoneItems(
            string zoneGroupID, string sortBy, SearchFilter searchFilter, int startIndex, int endIndex, out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchShippingZoneItemSignature,
                _cacheDependency,
                zoneGroupID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchShippingZoneItems(
                    zoneGroupID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchShippingZoneItemSignature,
                    result,
                    zoneGroupID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ShippingZoneItem>) result.First;
        }

        public IList<TableSchemaItem> GetShippingZoneItemTableSchemas()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                GetShippingZoneItemTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetShippingZoneItemTableSchemas();
                _dataCache.SetValue( GetShippingZoneItemTableSchemaSignature, list );
            }

            return list;
        }

        public ShippingZoneGroup Save( ShippingZoneGroup zoneGroup )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ShippingZoneGroup newZoneGroup = _repository.Save( zoneGroup );
            _dataCache.ClearRepositoryCache();
            return newZoneGroup;
        }

        public ShippingZoneGroup GetOne( string zoneGroupID )
        {
            ShippingZoneGroup result = (ShippingZoneGroup) _dataCache.GetValue( GetOneSignature, _cacheDependency, zoneGroupID );

            if (result == null)
            {
                result = _repository.GetOne( zoneGroupID );
                _dataCache.SetValue( GetOneSignature, result, zoneGroupID );
            }

            return result;
        }

        public void Delete( string zoneGroupID )
        {
            _repository.Delete( zoneGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ShippingZoneGroup> GetAll( string sortBy )
        {
            IList<ShippingZoneGroup> result = (IList<ShippingZoneGroup>) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy );

            if (result == null)
            {
                result = _repository.GetAll( sortBy );
                _dataCache.SetValue( GetAllSignature, result, sortBy );
            }

            return result;
        }

        public IList<ShippingZoneGroup> SearchShippingZoneGroups(
            string sortBy, SearchFilter searchFilter, int startIndex, int endIndex, out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchShippingZoneGroupSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
            );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchShippingZoneGroups(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchShippingZoneGroupSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ShippingZoneGroup>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchemas()
        {
            IList<TableSchemaItem> list
                = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchemas();
                _dataCache.SetValue( GetTableSchemaSignature, list
                );
            }

            return list;
        }

        #endregion
    }
}
