using System;
using Vevo.Base.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Stores;
using Vevo.Shared.SystemServices;

namespace Vevo.Domain
{
    public static class DataAccessContext
    {
        #region Private
        private static ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "Configuration", "ConfigurationValue" } );

        private static ConfigurationCollection _configurations;
        private static ApplicationSettings _applicationSettings;
        private static bool _configEnabled = false;
        private static IStoreRetriever _storeRetriever;

        #endregion


        #region Public Properties

        // This is the global configurations that will be set at the application startup.

        public static ConfigurationCollection Configurations
        {
            get
            {
                try
                {
                    if ( _cacheDependency.getHasChanged )
                    {
                        ClearConfigurationCache();
                    }
                }
                catch ( Exception ex )
                {
                    SaveLogFile.SaveLog( ex );
                }

                if ( _configurations == null || _configurations.Count == 0 )
                {
                    if ( !_configEnabled )
                        _configurations = new ConfigurationCollection();
                    else
                        _configurations = new ConfigurationCollection(
                            ConfigurationRepository.GetAll(),
                            StoreRetriever );
                }

                return _configurations;
            }
        }

        // This is the settings in App_Data\ApplicationSettings.config. This file contains 
        // system configurations that cannot be saved in the database.
        public static ApplicationSettings ApplicationSettings
        {
            get
            {
                if ( _applicationSettings == null )
                    _applicationSettings = new ApplicationSettings( new FileManager() );

                return _applicationSettings;
            }
        }

        public static IStoreRetriever StoreRetriever
        {
            get
            {
                if ( _storeRetriever == null )
                    _storeRetriever = new StoreRetriever();

                return _storeRetriever;
            }
        }

        public static IAdminRepository AdminRepository
        {
            get
            {
                return (IAdminRepository) DataAccessContextUtility.GetValue(
                    typeof(IAdminRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IAdminRepository)));
            }
        }


        public static IBannerRepository BannerRepository
        {
            get
            {
                return ( IBannerRepository ) DataAccessContextUtility.GetValue(
                    typeof(IBannerRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IBannerRepository ) ) );
            }
        }

        public static IBlogRepository BlogRepository
        {
            get
            {
                return ( IBlogRepository ) DataAccessContextUtility.GetValue(
                    typeof(IBlogRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IBlogRepository) ) );
            }
        }
        public static IBlogCommentRepository BlogCommentRepository
        {
            get
            {
                return (IBlogCommentRepository) DataAccessContextUtility.GetValue(
                    typeof(IBlogCommentRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IBlogCommentRepository ) ) );
            }
        }
        public static ICartItemRepository CartItemRepository
        {
            get
            {
                return ( ICartItemRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICartItemRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICartItemRepository ) ) );
            }
        }

        public static ICartRepository CartRepository
        {
            get
            {
                return ( ICartRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICartRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICartRepository ) ) );
            }
        }

        public static ICategoryRepository CategoryRepository
        {
            get
            {
                return ( ICategoryRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICategoryRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ICategoryRepository) ));
            }
        }

        public static IConfigurationRepository ConfigurationRepository
        {
            get
            {
                return ( IConfigurationRepository ) DataAccessContextUtility.GetValue(
                    typeof(IConfigurationRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IConfigurationRepository )) );
            }
        }

        public static IContentMenuItemRepository ContentMenuItemRepository
        {
            get
            {
                return ( IContentMenuItemRepository ) DataAccessContextUtility.GetValue(
                    typeof(IContentMenuItemRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IContentMenuItemRepository )) );
            }
        }

        public static IContentMenuRepository ContentMenuRepository
        {
            get
            {
                return ( IContentMenuRepository ) DataAccessContextUtility.GetValue(
                    typeof(IContentMenuRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IContentMenuRepository) ) );
            }
        }

        public static IContentRepository ContentRepository
        {
            get
            {
                return ( IContentRepository ) DataAccessContextUtility.GetValue(
                    typeof(IContentRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IContentRepository )) );
            }
        }


        public static ICountryRepository CountryRepository
        {
            get
            {
                return ( ICountryRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICountryRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ICountryRepository )) );
            }
        }

        public static ICouponRepository CouponRepository
        {
            get
            {
                return ( ICouponRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICouponRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICouponRepository ) ) );
            }
        }

        public static ICustomerRepository CustomerRepository
        {
            get
            {
                return ( ICustomerRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICustomerRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICustomerRepository ) ) );
            }
        }

        //public static ICustomerRewardPointRepository CustomerRewardPointRepository
        //{
        //    get
        //    {
        //        return ( ICustomerRewardPointRepository ) DataAccessContextUtility.GetValue(
        //            typeof(ICustomerRewardPointRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( ICustomerRewardPointRepository ) ) );
        //    }
        //}

        public static ICultureRepository CultureRepository
        {
            get
            {
                return ( ICultureRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICultureRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICultureRepository ) ) );
            }
        }

        public static ICurrencyRepository CurrencyRepository
        {
            get
            {
                return ( ICurrencyRepository ) DataAccessContextUtility.GetValue(
                    typeof(ICurrencyRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( ICurrencyRepository ) ) );
            }
        }

        public static IDepartmentRepository DepartmentRepository
        {
            get
            {
                return ( IDepartmentRepository ) DataAccessContextUtility.GetValue(
                    typeof(IDepartmentRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IDepartmentRepository ) ) );
            }
        }

        public static IDiscountGroupRepository DiscountGroupRepository
        {
            get
            {
                return ( IDiscountGroupRepository ) DataAccessContextUtility.GetValue(
                    typeof(IDiscountGroupRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IDiscountGroupRepository ) ) );
            }
        }

        public static IEmailTemplateDetailRepository EmailTemplateDetailRepository
        {
            get
            {
                return ( IEmailTemplateDetailRepository ) DataAccessContextUtility.GetValue(
                    typeof(IEmailTemplateDetailRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IEmailTemplateDetailRepository )) );
            }
        }

        //public static IEBayListRepository EBayListRepository
        //{
        //    get
        //    {
        //        return ( IEBayListRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IEBayListRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IEBayListRepository ) ) );
        //    }
        //}

        //public static IEBayTemplateRepository EBayTemplateRepository
        //{
        //    get
        //    {
        //        return ( IEBayTemplateRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IEBayTemplateRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IEBayTemplateRepository ) ) );
        //    }
        //}

        public static IGiftCertificateRepository GiftCertificateRepository
        {
            get
            {
                return ( IGiftCertificateRepository ) DataAccessContextUtility.GetValue(
                    typeof(IGiftCertificateRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IGiftCertificateRepository ) ) );
            }
        }

        //public static IGiftRegistryItemRepository GiftRegistryItemRepository
        //{
        //    get
        //    {
        //        return ( IGiftRegistryItemRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IGiftRegistryItemRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IGiftRegistryItemRepository ) ) );
        //    }
        //}

        //public static IGiftRegistryRepository GiftRegistryRepository
        //{
        //    get
        //    {
        //        return ( IGiftRegistryRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IGiftRegistryRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IGiftRegistryRepository ) ) );
        //    }
        //}

        public static IGoogleFeedMappingRepository GoogleFeedMappingRepository
        {
            get
            {
                return ( IGoogleFeedMappingRepository ) DataAccessContextUtility.GetValue(
                    typeof(IGoogleFeedMappingRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IGoogleFeedMappingRepository ) ) );
            }
        }

        public static IGoogleFeedTagRepository GoogleFeedTagRepository
        {
            get
            {
                return ( IGoogleFeedTagRepository ) DataAccessContextUtility.GetValue(
                    typeof(IGoogleFeedTagRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IGoogleFeedTagRepository ) ) );
            }
        }

        public static IGoogleFeedTagValueRepository GoogleFeedTagValueRepository
        {
            get
            {
                return ( IGoogleFeedTagValueRepository ) DataAccessContextUtility.GetValue(
                    typeof(IGoogleFeedTagValueRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IGoogleFeedTagValueRepository ) ) );
            }
        }

        public static IHelpRepository HelpRepository
        {
            get
            {
                return ( IHelpRepository ) DataAccessContextUtility.GetValue(
                    typeof(IHelpRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IHelpRepository ) ) );
            }
        }

        public static IManufacturerRepository ManufacturerRepository
        {
            get
            {
                return (IManufacturerRepository) DataAccessContextUtility.GetValue(
                    typeof(IManufacturerRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IManufacturerRepository ) ) );
            }
        }

        public static IBlogCategoryRepository BlogCategoryRepository
        {
            get
            {
                return (IBlogCategoryRepository) DataAccessContextUtility.GetValue(
                    typeof( IBlogCategoryRepository ),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IBlogCategoryRepository ) ) );
            }
        }
        public static INewsLetterRepository NewsLetterRepository
        {
            get
            {
                return ( INewsLetterRepository ) DataAccessContextUtility.GetValue(
                    typeof(INewsLetterRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( INewsLetterRepository ) ) );
            }
        }

        public static INewsRepository NewsRepository
        {
            get
            {
                return ( INewsRepository ) DataAccessContextUtility.GetValue(
                    typeof( INewsRepository ),
                    DataAccessContextUtility.GetCacheEnabled( typeof( INewsRepository ) ) );
            }
        }

        public static IOptionGroupRepository OptionGroupRepository
        {
            get
            {
                return ( IOptionGroupRepository ) DataAccessContextUtility.GetValue(
                    typeof(IOptionGroupRepository ),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IOptionGroupRepository ) ) );
            }
        }

        public static IOptionItemRepository OptionItemRepository
        {
            get
            {
                return ( IOptionItemRepository ) DataAccessContextUtility.GetValue(
                    typeof(IOptionItemRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IOptionItemRepository ) ) );
            }
        }

        public static IOrderRepository OrderRepository
        {
            get
            {
                return ( IOrderRepository ) DataAccessContextUtility.GetValue(
                    typeof(IOrderRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IOrderRepository ) ) );
            }
        }

        public static IOrderItemRepository OrderItemRepository
        {
            get
            {
                return ( IOrderItemRepository ) DataAccessContextUtility.GetValue(
                    typeof(IOrderItemRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IOrderItemRepository ) ) );
            }
        }

        public static IPaymentLogRepository PaymentLogRepository
        {
            get
            {
                return ( IPaymentLogRepository ) DataAccessContextUtility.GetValue(
                    typeof(IPaymentLogRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IPaymentLogRepository ) ) );
            }
        }

        public static IPaymentOptionRepository PaymentOptionRepository
        {
            get
            {
                return ( IPaymentOptionRepository ) DataAccessContextUtility.GetValue(
                    typeof(IPaymentOptionRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IPaymentOptionRepository ) ) );
            }
        }

        public static IPayPalIpnRepository PayPalIpnRepository
        {
            get
            {
                return ( IPayPalIpnRepository ) DataAccessContextUtility.GetValue(
                    typeof(IPayPalIpnRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IPayPalIpnRepository ) ) );
            }
        }

        public static IProductKitGroupRepository ProductKitGroupRepository
        {
            get
            {
                return ( IProductKitGroupRepository ) DataAccessContextUtility.GetValue(
                    typeof(IProductKitGroupRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IProductKitGroupRepository) ) );
            }
        }

        public static IProductRepository ProductRepository
        {
            get
            {
                return ( IProductRepository ) DataAccessContextUtility.GetValue(
                    typeof(IProductRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IProductRepository ) ) );
            }
        }

        //public static IPromotionGroupRepository PromotionGroupRepository
        //{
        //    get
        //    {
        //        return ( IPromotionGroupRepository ) DataAccessContextUtility.GetValue(
        //            typeof( IPromotionGroupRepository ),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IPromotionGroupRepository ) ) );
        //    }
        //}

        //public static IPromotionProductRepository PromotionProductRepository
        //{
        //    get
        //    {
        //        return ( IPromotionProductRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IPromotionProductRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IPromotionProductRepository ) ) );
        //    }
        //}

        //public static IPromotionSubGroupRepository PromotionSubGroupRepository
        //{
        //    get
        //    {
        //        return ( IPromotionSubGroupRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IPromotionSubGroupRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IPromotionSubGroupRepository ) ) );
        //    }
        //}

        //public static IQuickBooksLogRepository QuickBooksLogRepository
        //{
        //    get
        //    {
        //        return ( IQuickBooksLogRepository ) DataAccessContextUtility.GetValue(
        //            typeof(IQuickBooksLogRepository),
        //            DataAccessContextUtility.GetCacheEnabled( typeof( IQuickBooksLogRepository ) ) );
        //    }
        //}

        public static IRecurringProfileRepository RecurringProfileRepository
        {
            get
            {
                return ( IRecurringProfileRepository ) DataAccessContextUtility.GetValue(
                    typeof(IRecurringProfileRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IRecurringProfileRepository ) ) );
            }
        }

        public static IRmaRepository RmaRepository
        {
            get
            {
                return ( IRmaRepository ) DataAccessContextUtility.GetValue(
                    typeof(IRmaRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IRmaRepository ) ) );
            }
        }

        public static IRmaActionRepository RmaActionRepository
        {
            get
            {
                return ( IRmaActionRepository ) DataAccessContextUtility.GetValue(
                    typeof(IRmaActionRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IRmaActionRepository ) ) );
            }
        }

        public static IShippingOptionRepository ShippingOptionRepository
        {
            get
            {
                return ( IShippingOptionRepository ) DataAccessContextUtility.GetValue(
                    typeof(IShippingOptionRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IShippingOptionRepository ) ) );
            }
        }

        public static IShippingOptionTypeRepository ShippingOptionTypeRepository
        {
            get
            {
                return ( IShippingOptionTypeRepository ) DataAccessContextUtility.GetValue(
                    typeof(IShippingOptionTypeRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IShippingOptionTypeRepository )) );
            }
        }

        public static IShippingWeightRateRepository ShippingWeightRateRepository
        {
            get
            {
                return ( IShippingWeightRateRepository ) DataAccessContextUtility.GetValue(
                    typeof(IShippingWeightRateRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IShippingWeightRateRepository )) );
            }
        }

        public static IShippingOrderTotalRateRepository ShippingOrderTotalRateRepository
        {
            get
            {
                return ( IShippingOrderTotalRateRepository ) DataAccessContextUtility.GetValue(
                    typeof(IShippingOrderTotalRateRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(IShippingOrderTotalRateRepository )) );
            }
        }

        public static IShippingZoneGroupRepository ShippingZoneGroupRepository
        {
            get
            {
                return ( IShippingZoneGroupRepository ) DataAccessContextUtility.GetValue(
                    typeof(IShippingZoneGroupRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IShippingZoneGroupRepository ) ) );
            }
        }

        public static ISpecificationGroupRepository SpecificationGroupRepository
        {
            get
            {
                return ( ISpecificationGroupRepository ) DataAccessContextUtility.GetValue(
                    typeof(ISpecificationGroupRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ISpecificationGroupRepository) ) );
            }
        }

        public static ISpecificationItemRepository SpecificationItemRepository
        {
            get
            {
                return ( ISpecificationItemRepository ) DataAccessContextUtility.GetValue(
                    typeof(ISpecificationItemRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ISpecificationItemRepository) ) );
            }
        }

        public static ISpecificationItemValueRepository SpecificationItemValueRepository
        {
            get
            {
                return ( ISpecificationItemValueRepository ) DataAccessContextUtility.GetValue(
                    typeof(ISpecificationItemValueRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ISpecificationItemValueRepository) ) );
            }
        }

        public static IStateRepository StateRepository
        {
            get
            {
                return ( IStateRepository ) DataAccessContextUtility.GetValue(
                    typeof(IStateRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IStateRepository ) ) );
            }
        }

        public static IStoreRepository StoreRepository
        {
            get
            {
                return ( IStoreRepository ) DataAccessContextUtility.GetValue(
                    typeof(IStoreRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IStoreRepository ) ) );
            }
        }

        public static ITaxClassRepository TaxClassRepository
        {
            get
            {
                return ( ITaxClassRepository ) DataAccessContextUtility.GetValue(
                    typeof(ITaxClassRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof(ITaxClassRepository) ) );
            }
        }

        public static IWishListRepository WishListRepository
        {
            get
            {
                return ( IWishListRepository ) DataAccessContextUtility.GetValue(
                    typeof(IWishListRepository),
                    DataAccessContextUtility.GetCacheEnabled( typeof( IWishListRepository ) ) );
            }
        }

        #endregion


        #region Public Methods

        public static void ClearConfigurationCache()
        {
            _configurations = null;
            _applicationSettings = null;
        }

        public static void SetUpRepository( System.Type type, object cached, object nonCached )
        {
            DataAccessContextUtility.SetValue( type, true, cached );
            DataAccessContextUtility.SetValue( type, false, nonCached );
        }

        public static void EnableConfigurations( bool enabled )
        {
            _configEnabled = enabled;
            ClearConfigurationCache();
        }

        public static void SetStoreRetriever( IStoreRetriever storeRetriever )
        {
            _storeRetriever = storeRetriever;
        }
        #endregion
    }
}
