﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Facebook;
using Vevo.Domain.Users;
using Vevo.Shared.WebUI;

namespace Vevo.WebUI.Facebook
{
    public class FacebookRegister : FacebookConnect
    {
        public FacebookRegister()
            : base()
        {
            _facebookOAuthClient.RedirectUri = new UriBuilder( UrlPath.StorefrontUrl + "Register.aspx" ).Uri;
        }

        public FacebookUser GetUserInfo( string code )
        {
            try
            {
                var parameters = new Dictionary<string, object>();
                parameters.Add( "permissions", "offline_access" );

                JsonObject tokenObj = (JsonObject) _facebookOAuthClient.ExchangeCodeForAccessToken( code, parameters );
                
                if (!tokenObj.ContainsKey( "access_token" ))
                    return NullFacebookUser.Instance;
                FacebookClient fb = new FacebookClient( tokenObj["access_token"].ToString() );
                JsonObject user = (JsonObject) fb.Get( "/me" );
                FacebookUser fbuser = new FacebookUser();
                if (user.ContainsKey( "id" )) fbuser.UserID = user["id"].ToString();
                if (user.ContainsKey( "first_name" )) fbuser.FirstName = user["first_name"].ToString();
                if (user.ContainsKey( "last_name" )) fbuser.LastName = user["last_name"].ToString();
                if (user.ContainsKey( "email" )) fbuser.Email = user["email"].ToString();

                if (user.ContainsKey( "location" ))
                {
                    JsonObject location = (JsonObject) user["location"];
                    if (user.ContainsKey( "name" ))
                    {                        
                        string[] locations = location["name"].ToString().Split( ",".ToCharArray() );
                        if (locations.Length > 0)
                        {
                            fbuser.City = locations[0];
                            if (locations.Length > 1)
                                fbuser.Country = locations[1].Trim();
                        }
                    }
                }
                if (user.ContainsKey( "work" ))
                {
                    JsonArray works = (JsonArray) user["work"];
                    foreach (JsonObject work in works)
                    {
                        if (!work.ContainsKey( "end_date" ))
                        {
                            if (work.ContainsKey( "employer" ))
                            {
                                JsonObject employer = (JsonObject) work["employer"];
                                fbuser.Company = employer["name"].ToString();
                                break;
                            }
                        }
                    }
                }

                return fbuser;
            }
            catch
            {
                return NullFacebookUser.Instance;
            }
        }
    }
}
